@@ -2,6 +2,7 @@ RSpec::Support.require_rspec_support 'recursive_const_methods'
 
 module RSpec
   module Core
+    # rubocop:disable Metrics/ClassLength
     # ExampleGroup and {Example} are the main structural elements of
     # rspec-core. Consider this example:
     #
@@ -32,6 +33,8 @@ module RSpec
       include Pending
       extend SharedExampleGroup
 
+      # Define a singleton method for the singleton class (remove the method if
+      # it's already been defined).
       # @private
       def self.idempotently_define_singleton_method(name, &definition)
         (class << self; self; end).module_exec do
@@ -142,9 +145,7 @@ module RSpec
           options.update(:skip => RSpec::Core::Pending::NOT_YET_IMPLEMENTED) unless block
           options.update(extra_options)
 
-          example = RSpec::Core::Example.new(self, desc, options, block)
-          examples << example
-          example
+          RSpec::Core::Example.new(self, desc, options, block)
         end
       end
 
@@ -234,27 +235,27 @@ module RSpec
           thread_data = RSpec::Support.thread_local_data
           top_level   = self == ExampleGroup
 
-          if top_level
-            if thread_data[:in_example_group]
-              raise "Creating an isolated context from within a context is " \
-                    "not allowed. Change `RSpec.#{name}` to `#{name}` or " \
-                    "move this to a top-level scope."
+          registration_collection =
+            if top_level
+              if thread_data[:in_example_group]
+                raise "Creating an isolated context from within a context is " \
+                      "not allowed. Change `RSpec.#{name}` to `#{name}` or " \
+                      "move this to a top-level scope."
+              end
+
+              thread_data[:in_example_group] = true
+              RSpec.world.example_groups
+            else
+              children
             end
 
-            thread_data[:in_example_group] = true
-          end
-
           begin
-
             description = args.shift
             combined_metadata = metadata.dup
             combined_metadata.merge!(args.pop) if args.last.is_a? Hash
             args << combined_metadata
 
-            subclass(self, description, args, &example_group_block).tap do |child|
-              children << child
-            end
-
+            subclass(self, description, args, registration_collection, &example_group_block)
           ensure
             thread_data.delete(:in_example_group) if top_level
           end
@@ -340,26 +341,47 @@ module RSpec
         find_and_eval_shared("examples", name, caller.first, *args, &block)
       end
 
+      # Clear memoized values when adding/removing examples
+      # @private
+      def self.reset_memoized
+        @descendant_filtered_examples = nil
+        @_descendants = nil
+        @parent_groups = nil
+        @declaration_locations = nil
+      end
+
+      # Adds an example to the example group
+      def self.add_example(example)
+        reset_memoized
+        examples << example
+      end
+
+      # Removes an example from the example group
+      def self.remove_example(example)
+        reset_memoized
+        examples.delete example
+      end
+
       # @private
       def self.find_and_eval_shared(label, name, inclusion_location, *args, &customization_block)
-        shared_block = RSpec.world.shared_example_group_registry.find(parent_groups, name)
+        shared_module = RSpec.world.shared_example_group_registry.find(parent_groups, name)
 
-        unless shared_block
+        unless shared_module
           raise ArgumentError, "Could not find shared #{label} #{name.inspect}"
         end
 
-        SharedExampleGroupInclusionStackFrame.with_frame(name, Metadata.relative_path(inclusion_location)) do
-          module_exec(*args, &shared_block)
-          module_exec(&customization_block) if customization_block
-        end
+        shared_module.include_in(
+          self, Metadata.relative_path(inclusion_location),
+          args, customization_block
+        )
       end
 
       # @!endgroup
 
       # @private
-      def self.subclass(parent, description, args, &example_group_block)
+      def self.subclass(parent, description, args, registration_collection, &example_group_block)
         subclass = Class.new(parent)
-        subclass.set_it_up(description, *args, &example_group_block)
+        subclass.set_it_up(description, args, registration_collection, &example_group_block)
         subclass.module_exec(&example_group_block) if example_group_block
 
         # The LetDefinitions module must be included _after_ other modules
@@ -372,7 +394,7 @@ module RSpec
       end
 
       # @private
-      def self.set_it_up(description, *args, &example_group_block)
+      def self.set_it_up(description, args, registration_collection, &example_group_block)
         # Ruby 1.9 has a bug that can lead to infinite recursion and a
         # SystemStackError if you include a module in a superclass after
         # including it in a subclass: https://gist.github.com/845896
@@ -383,6 +405,16 @@ module RSpec
         # here.
         ensure_example_groups_are_configured
 
+        # Register the example with the group before creating the metadata hash.
+        # This is necessary since creating the metadata hash triggers
+        # `when_first_matching_example_defined` callbacks, in which users can
+        # load RSpec support code which defines hooks. For that to work, the
+        # examples and example groups must be registered at the time the
+        # support code is called or be defined afterwards.
+        # Begin defined beforehand but registered afterwards causes hooks to
+        # not be applied where they should.
+        registration_collection << self
+
         user_metadata = Metadata.build_hash_from(args)
 
         @metadata = Metadata::ExampleGroupHash.create(
@@ -392,7 +424,8 @@ module RSpec
         )
         ExampleGroups.assign_const(self)
 
-        hooks.register_globals(self, RSpec.configuration.hooks)
+        @currently_executing_a_context_hook = false
+
         RSpec.configuration.configure_group(self)
       end
 
@@ -420,10 +453,19 @@ module RSpec
       # @private
       def self.next_runnable_index_for(file)
         if self == ExampleGroup
-          RSpec.world.num_example_groups_defined_in(file)
+          # We add 1 so the ids start at 1 instead of 0. This is
+          # necessary for this branch (but not for the other one)
+          # because we register examples and groups with the
+          # `children` and `examples` collection BEFORE this
+          # method is called as part of metadata hash creation,
+          # but the example group is recorded with
+          # `RSpec.world.example_group_counts_by_spec_file` AFTER
+          # the metadata hash is created and the group is returned
+          # to the caller.
+          RSpec.world.num_example_groups_defined_in(file) + 1
         else
           children.count + examples.count
-        end + 1
+        end
       end
 
       # @private
@@ -464,15 +506,24 @@ module RSpec
         end
       end
 
+      # Returns true if a `before(:context)` or `after(:context)`
+      # hook is currently executing.
+      def self.currently_executing_a_context_hook?
+        @currently_executing_a_context_hook
+      end
+
       # @private
       def self.run_before_context_hooks(example_group_instance)
         set_ivars(example_group_instance, superclass_before_context_ivars)
 
+        @currently_executing_a_context_hook = true
+
         ContextHookMemoized::Before.isolate_for_context_hook(example_group_instance) do
           hooks.run(:before, :context, example_group_instance)
         end
       ensure
         store_before_context_ivars(example_group_instance)
+        @currently_executing_a_context_hook = false
       end
 
       if RUBY_VERSION.to_f >= 1.9
@@ -503,11 +554,14 @@ module RSpec
       def self.run_after_context_hooks(example_group_instance)
         set_ivars(example_group_instance, before_context_ivars)
 
+        @currently_executing_a_context_hook = true
+
         ContextHookMemoized::After.isolate_for_context_hook(example_group_instance) do
           hooks.run(:after, :context, example_group_instance)
         end
       ensure
         before_context_ivars.clear
+        @currently_executing_a_context_hook = false
       end
 
       # Runs all the examples in this group.
@@ -524,9 +578,9 @@ module RSpec
         rescue Pending::SkipDeclaredInExample => ex
           for_filtered_examples(reporter) { |example| example.skip_with_exception(reporter, ex) }
           true
-        rescue Exception => ex
-          RSpec.world.wants_to_quit = true if fail_fast?
+        rescue Support::AllExceptionsExceptOnesWeMustNotRescue => ex
           for_filtered_examples(reporter) { |example| example.fail_with_exception(reporter, ex) }
+          RSpec.world.wants_to_quit = true if reporter.fail_fast_limit_met?
           false
         ensure
           run_after_context_hooks(new('after(:context) hook')) if should_run_context_hooks
@@ -557,7 +611,9 @@ module RSpec
           instance = new(example.inspect_output)
           set_ivars(instance, before_context_ivars)
           succeeded = example.run(instance, reporter)
-          RSpec.world.wants_to_quit = true if fail_fast? && !succeeded
+          if !succeeded && reporter.fail_fast_limit_met?
+            RSpec.world.wants_to_quit = true
+          end
           succeeded
         end.all?
       end
@@ -575,15 +631,10 @@ module RSpec
       end
 
       # @private
-      def self.fail_fast?
-        RSpec.configuration.fail_fast?
-      end
-
-      # @private
-      def self.declaration_line_numbers
-        @declaration_line_numbers ||= [metadata[:line_number]] +
-          examples.map { |e| e.metadata[:line_number] } +
-          FlatMap.flat_map(children, &:declaration_line_numbers)
+      def self.declaration_locations
+        @declaration_locations ||= [Metadata.location_tuple_from(metadata)] +
+          examples.map { |e| Metadata.location_tuple_from(e.metadata) } +
+          FlatMap.flat_map(children, &:declaration_locations)
       end
 
       # @return [String] the unique id of this example group. Pass
@@ -638,6 +689,17 @@ module RSpec
         # :nocov:
       end
 
+      # @private
+      def self.update_inherited_metadata(updates)
+        metadata.update(updates) do |_key, existing_group_value, _new_inherited_value|
+          existing_group_value
+        end
+
+        RSpec.configuration.configure_group(self)
+        examples.each { |ex| ex.update_inherited_metadata(updates) }
+        children.each { |group| group.update_inherited_metadata(updates) }
+      end
+
       # Raised when an RSpec API is called in the wrong scope, such as `before`
       # being called from within an example rather than from within an example
       # group block.
@@ -671,6 +733,7 @@ module RSpec
         super
       end
     end
+    # rubocop:enable Metrics/ClassLength
 
     # @private
     # Unnamed example group used by `SuiteHookContext`.
