@@ -106,6 +106,11 @@ module RSpec
         "#{metadata[:rerun_file_path]}[#{metadata[:scoped_id]}]"
       end
 
+      # @private
+      def self.location_tuple_from(metadata)
+        [metadata[:absolute_file_path], metadata[:line_number]]
+      end
+
       # @private
       # Used internally to populate metadata hashes with computed keys
       # managed by RSpec.
@@ -123,7 +128,6 @@ module RSpec
         def populate
           ensure_valid_user_keys
 
-          metadata[:execution_result] = Example::ExecutionResult.new
           metadata[:block]            = block
           metadata[:description_args] = description_args
           metadata[:description]      = build_description_from(*metadata[:description_args])
@@ -214,6 +218,7 @@ module RSpec
           end)
           group_metadata.update(example_metadata)
 
+          example_metadata[:execution_result] = Example::ExecutionResult.new
           example_metadata[:example_group] = group_metadata
           example_metadata[:shared_group_inclusion_backtrace] = SharedExampleGroupInclusionStackFrame.current_backtrace
           example_metadata.delete(:parent_example_group)
