@@ -1,7 +1,6 @@
 RSpec::Support.require_rspec_core "formatters/exception_presenter"
 RSpec::Support.require_rspec_core "formatters/helpers"
 RSpec::Support.require_rspec_core "shell_escape"
-RSpec::Support.require_rspec_support "encoded_string"
 
 module RSpec::Core
   # Notifications are value objects passed to formatters to provide them
@@ -10,6 +9,7 @@ module RSpec::Core
     # @private
     module NullColorizer
       module_function
+
       def wrap(line, _code_or_symbol)
         line
       end
@@ -202,7 +202,7 @@ module RSpec::Core
 
     private
 
-      def initialize(example, exception_presenter=Formatters::ExceptionPresenter.new(example.execution_result.exception, example))
+      def initialize(example, exception_presenter)
         @exception_presenter = exception_presenter
         super(example)
       end
@@ -281,7 +281,8 @@ module RSpec::Core
     # @attr pending_examples [Array<RSpec::Core::Example>] the pending examples
     # @attr load_time [Float] the number of seconds taken to boot RSpec
     #                         and load the spec files
-    SummaryNotification = Struct.new(:duration, :examples, :failed_examples, :pending_examples, :load_time)
+    SummaryNotification = Struct.new(:duration, :examples, :failed_examples,
+                                     :pending_examples, :load_time)
     class SummaryNotification
       # @api
       # @return [Fixnum] the number of examples run
